#!/usr/bin/bash
#
# PrivX Carrier post install script.
#
# Run after installing the rpm package.
#

# Parse arguments
for arg in "$@"
do
  case $arg in
    --log-to-file)
    logfile=/var/log/privx-install.log
    exec 3>&1 1>>${logfile} 2>&1
    echo "-----"
    echo "PrivX Carrier install "`date`
    echo
    ;;

    --request-cert)
    request_cert="yes"
    ;;

    *)
    echo "unknown argument $arg"
    exit 1
    ;;
  esac
done

# Check if need to skip automatic postinstall
case $SKIP_POSTINSTALL in
true|TRUE|1)
  echo
  echo "Skipping automatic post installation steps"
  echo
  echo "Please make sure to run the /opt/privx/scripts/carrier-postinstall.sh manually."
  echo
  exit 0
  ;;
*)
  ;;
esac

# Check if the disk space is low
echo "Checking if there is enough free disk space:"
availableSpace=$(df -BG --output=avail / | grep '[0-9]' | sed 's/.$//')
if [ "$availableSpace" -le 5 ]
then
  echo "[WARNING]: Disk Space is low! Make sure you have more than 5GB of Disk Storage available to prevent corrupted Docker Images"
else
  echo "Sufficient amount of free disk space detected"
fi

# Check that configuration file exists
echo "Checking carrier configuration file:"
carrierconfig="/opt/privx/etc/carrier-config.toml"
if [ ! -r "$carrierconfig" ]
then
  echo "$carrierconfig missing."
  echo
  echo "Download carrier configuration from PrivX UI,"
  echo "copy it to $carrierconfig"
  echo "and re-run /opt/privx/scripts/carrier-postinstall.sh"
  echo
  exit 0
fi

# Set docker env
export $(grep privx_docker_binary_path /opt/privx/etc/carrier-config.toml | tr -d ' "')
if [ ! -r "$privx_docker_binary_path" ]
then
  echo "Docker binary missing: $privx_docker_binary_path"
  echo "Please check the PrivX-Carrier documentation and install Docker accordingly."
  exit 0
fi

groupadd -f docker
usermod -aG docker privx
chgrp docker $privx_docker_binary_path

chmod 750 /opt/privx
chgrp -R privx /opt/privx

# Create home dir for privx user for Podman (if missing)
mkhomedir_helper privx

# Remove all permissions from other to keep credentials safe
chown root:privx $carrierconfig
chmod 0640 $carrierconfig
echo "done"

# Web container file transfer dir
mkdir -p /opt/privx/filetransfer
chmod 0771 /opt/privx/filetransfer
chown -R privx:docker /opt/privx/filetransfer

# Stop carrier
echo "Checking carrier status:"
if systemctl is-active privx-carrier
then
  echo "Stopping carrier:"
  systemctl stop privx-carrier
  echo "done"
fi
# In case systemd got confused after config change on upgrade:
if [[ `pidof privx-carrier` ]]; then
  kill `pidof privx-carrier`
fi

# Register carrier to server
echo "Registering carrier:"

if [ -n "$request_cert" ]
then
  # Move existing cert and private key to request a new cert
  certfile=`sed -n 's;certificate_file\s*=\s*"\(.*\)";\1;p' $carrierconfig`
  keyfile=`sed -n 's;private_key_file\s*=\s*"\(.*\)";\1;p' $carrierconfig`
  if [ -n "$certfile" ]; then certfile="/opt/privx/carrier/carrier.crt"; fi
  if [ -n "$keyfile" ]; then keyfile="/opt/privx/carrier/carrier.key"; fi
  if [ -e "$certfile" ]; then mv -f "$certfile" "$certfile.bak"; fi
  if [ -e "$keyfile" ]; then mv -f "$keyfile" "$keyfile.bak"; fi
fi

su privx -c "/opt/privx/bin/privx-carrier -register"
status=$?

if [ $status -ne 0 ]
then
  if [ -e "$certfile.bak" ]; then mv -f "$certfile.bak" "$certfile"; fi
  if [ -e "$keyfile.bak" ]; then mv -f "$keyfile.bak" "$keyfile"; fi

  echo "Registration failed."
  echo "Check the following:"
  echo "  - Configuration in $carrierconfig is correct"
  echo "  - PrivX server is reachable"
  echo "  - Carrier is not already registered in PrivX UI"
  echo "Re-run /opt/privx/scripts/carrier-postinstall.sh after fixing the problem."
  echo
  exit $status
fi
rm -f "$certfile.bak" "$keyfile.bak"
echo "done"

# Enable docker
echo "Enabling docker:"
systemctl enable docker || systemctl enable podman
echo "done"

# Enable PrivX Carrier
echo "Enabling privx-carrier:"
systemctl enable privx-carrier
echo "done"

# Start docker
echo "Starting docker:"
systemctl restart docker || systemctl restart podman
echo "done"

# Start PrivX Carrier
echo "Starting privx-carrier:"
systemctl restart privx-carrier
echo "done"

exit 0
